#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <pthread.h>


#include "libthrd.h"



int initialisationServeur(char* service)
{
	int sock_fd;

	struct addrinfo precisions, *resultat=NULL, *origine; // On stocke dans précisions nos besoins pour le socket, dans resultat l'adresse qui respectera les reqêtes

	memset(&precisions, 0, sizeof precisions);
	precisions.ai_family = AF_UNSPEC;
	precisions.ai_socktype = SOCK_STREAM;
	precisions.ai_flags = AI_PASSIVE;
	
	if(getaddrinfo(NULL, service, &precisions, &origine)<0) { fprintf(stderr, "Erreur getaddrinfo\n"); return(-1); }
	
	int n=0;
	
	for(struct addrinfo* i=origine; i!=NULL && resultat==NULL; i=i->ai_next)
	{
		printf("%d - ", n);
		if(i->ai_family==AF_INET)
		{
			resultat=i;
			printf("Test : %d", origine->ai_addrlen);
		}

		printf("\n");	
		n++;
	}

	struct sockaddr_in* test=(struct sockaddr_in*)(resultat->ai_addr);
	printf("Addr : %x\n", test->sin_addr.s_addr);
	if((sock_fd=socket(resultat->ai_family, resultat->ai_socktype, resultat->ai_protocol))<0) { fprintf(stderr, "Erreur socket\n"); return(-1); }

	int val=1;
	
	if(setsockopt(sock_fd, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val))<0) { fprintf(stderr, "Erreur setsockopt\n"); return(-1); }

	if(setsockopt(sock_fd, SOL_SOCKET, SO_BROADCAST, &val, sizeof(val))<0) { fprintf(stderr, "Erreur setsockopt\n"); return(-1); }


	if(bind(sock_fd, resultat->ai_addr, resultat->ai_addrlen)) { fprintf(stderr, "Erreur bind\n"); return(-1); }

	if(listen(sock_fd, 20) < 0) { fprintf(stderr, "Error listen server socket\n"); return (-1); }
	
	freeaddrinfo(origine);
	
	return sock_fd;
}




int boucleServeur(int socket, void(* fctConnex)(int))
{
	int sock_dial;

	//On utilisera la structure Arg_Thread (définie dans libthrd.h) qui comprend la socket de dialogue et le tableau pointeurs vers une structure donnant les infos des interfaces.
	//En effet, cette structure d'informations d'interface permet d'être utilisée comme une base de données, comprenant l'adresse de l'interface et son statut
	//Les threads partageant la mémoire, ces infos seront partagées.
	//Afin de gérer les requêtes, il est nécessaire d'instaurer un mutex.
	//En effet, la ressource interfaces ne doit être accessible que par un seul thread à la fois

	Arg_Thread arg_sock_interf;

	//On initialise les interfaces
	for(int i=0; i<20; i++)
		arg_sock_interf.interfaces[i]=NULL;


	//On initialiser le mutex
	pthread_mutex_init(&(arg_sock_interf.requete_mutex), NULL);

	printf("------- Début boucle serveur TCP -------\n\n");
	
	while(1)	
	{
		printf("-- boucle --\n");
		if((sock_dial=accept(socket, NULL, NULL)) < 0){ fprintf(stderr, "Error accept dialogue\n"); return -1; }
		printf("Dialogue ACCEPTED\n");

		arg_sock_interf.socket=sock_dial;

		lanceThread((void*)fctConnex, (void*)&arg_sock_interf, sizeof(arg_sock_interf));
	}

	pthread_mutex_destroy(&(arg_sock_interf.requete_mutex));
}

